// Copyright 2005-2011, FreeHEP.
package hep.wired.cut;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 * Adapter for implementing {@link Condition}.
 *
 * @author Mark Donszelmann
 * @author Dmitry Onoprienko
 * @version $Id: AbstractCondition.java 13845 2011-06-09 00:14:38Z onoprien $
 */
abstract public class AbstractCondition implements Condition {

// -- Private parts : ----------------------------------------------------------

  protected ConditionDefinition _definition;
  private Cut _cut;
  protected boolean _enabled;
  protected boolean _inverted;
  protected boolean _set;

  private ChangeListener _listener;
  private ChangeEvent _defaultEvent;

// -- Construction and initialization : ----------------------------------------

  public AbstractCondition(ConditionDefinition definition, boolean enabled, boolean inverted) {
    _definition = definition;
    _enabled = enabled;
    _inverted = inverted;
    _set = false;
    _defaultEvent = new ChangeEvent(this);
  }

// -- Getters : ----------------------------------------------------------------

  /** Returns the Cut this Condition belongs to. */
  public Cut getCut() {
    return _cut;
  }

  /** Returns <tt>true</tt> is this condition is enabled. */
  public boolean isEnabled() {
    return _enabled;
  }

  /** Returns <tt>true</tt> is this condition is inverted. */
  public boolean isInverted() {
    return _inverted;
  }

  /** Returns <tt>ConditionDefinition</tt> associated with this condition. */
  public ConditionDefinition getDefinition() {
    return _definition;
  }


// -- Setters : ----------------------------------------------------------------

  /** Sets the Cut this Condition belongs to. */
  public void setCut(Cut cut) {
    _cut = cut;
  }

  /** Enables or disables this condition. */
  public void setEnabled(boolean state) {
    _enabled = state;
    fireStateChanged();
  }

  /** Sets inversion state of this condition. */
  public void setInverted(boolean state) {
    _inverted = state;
    fireStateChanged();
  }

  /**
   * Resets condition to its default state.
   * For most kinds of conditions, that means nothing (everything) is cut if the condition
   * is not (is) inverted. Boolean conditions are an exception since their state is fixed.
   */
  public void reset() {
    set(getDefinition().getDomain());
    _set = false;
    fireStateChanged();
  }


// -- Sending events : ---------------------------------------------------------

  /** Makes the Cut associated with this Condition fire a ChangeEvent with this Condition as a source. */
  protected void fireStateChanged() {
    if (_cut != null) _cut.fireStateChanged(_defaultEvent);
  }

// -- Overriding Object : ------------------------------------------------------

  public String toString() {
    return _definition.getName();
  }

// -----------------------------------------------------------------------------
}
