// Copyright 2005-2011, FreeHEP.
package hep.wired.cut;

import org.freehep.util.Value;

/**
 * {@link Condition} based on comparison with boolean value.
 *
 * @author Mark Donszelmann
 * @author Dmitry Onoprienko
 * @version $Id: BooleanCondition.java 13824 2011-05-04 18:30:02Z onoprien $
 */
public class BooleanCondition extends AbstractCondition {

// -- Private parts : ----------------------------------------------------------

// -- Construction and initialization : ----------------------------------------

  public BooleanCondition(ConditionDefinition definition) {
    this(definition, false, false);
  }

  public BooleanCondition(ConditionDefinition definition, boolean enabled, boolean inverted) {
    super(definition, enabled, inverted);
  }

// -- Getters : ----------------------------------------------------------------

  /** Returns status string describing what attribute values satisfy this condition in its current state. */
  public String getStatus() {
    return _inverted ?  "x = false" : "x = true";
  }

  /** Returns <tt>null</tt> since boolean condition criteria are unmodifiable. */
  public Object get() {
    return null;
  }

// -- Applying condition : -----------------------------------------------------

  /** Returns <tt>true</tt> is the value satisfies this condition. */
  public boolean pass(Value value) {
    return _inverted ? !value.getBoolean() :  value.getBoolean();
  }

// -- Setters : ----------------------------------------------------------------

  /** Does nothing since boolean condition criteria are unmodifiable. */
  public void set(Object value) {}

// -----------------------------------------------------------------------------
}
