// Copyright 2005-2011, FreeHEP.
package hep.wired.cut;

import org.freehep.util.Value;

/**
 * Definition for conditions based on boolean value.
 * Domain if fixed so setters do nothing and getter returns null.
 *
 * @author Dmitry Onoprienko
 * @version $Id: BooleanConditionDefinition.java 13824 2011-05-04 18:30:02Z onoprien $
 */
public class BooleanConditionDefinition extends AbstractConditionDefinition {

// -- Construction and initialization : ----------------------------------------

  public BooleanConditionDefinition(String name) {
    super(name);
  }

  public BooleanConditionDefinition(String name, String description) {
    super(name, null, description);
  }

  
// -- Type : -------------------------------------------------------------------

  /** Returns the type of this condition. */
  public Type getType() {
    return Type.BOOLEAN;
  }


// -- Creating Conditions : ----------------------------------------------------

  /** Returns a new {@link Condition} object linked to this ConditionDefinition. */
  public BooleanCondition createCondition(boolean enabled, boolean inverted) {
    BooleanCondition c = new BooleanCondition(this, enabled, inverted);
    _conditions.add(c);
    return c;
  }


// -----------------------------------------------------------------------------

  /**
   * Sets the domain of possible values for the condition.
   * Domain is fixed for boolean conditions so this method does nothing.
   */
  public void setDomain(Object domain) {}

  /**
   * Returns the domain of possible values for the condition.
   * Domain is fixed for boolean conditions so this method returns null.
   */
  public Object getDomain() {return null;}

  /**
   * Updates the domain to ensure it contains the specified value.
   * Domain is fixed for boolean conditions so this method does nothing.
   */
  public void addToDomain(Value value) {}

}
