package hep.wired.cut;

import java.util.Collection;
import java.util.Set;

/**
 * Interface to be implemented by the component that manages cuts and displays editors.
 *
 * @author Dmitry Onoprienko
 * @version $Id: $
 */
public interface CutManager {

  /** Displays pre-initialized condition editor. */
  void showConditionEditor(ConditionEditor editor);

  /** Returns a set of all known condition definitions. */
  Collection<ConditionDefinition> getDefinitions();

  /**
   * Returns ConditionDefinition with the specified name.
   * Returns <tt>null</tt> in no such condition is known to this CutManager.
   */
  ConditionDefinition getDefinition(String name);

  /** Returns a set of condition definitions applicable to the currently loaded record. */
  Set<ConditionDefinition> getCurrentDefinitions();

  /** Returns a set of condition definitions applicable to the currently loaded record and the specified cut. */
  Set<ConditionDefinition> getCurrentDefinitions(Cut cut);
}
