// Copyright 2005-2011, FreeHEP.
package hep.wired.cut;

import org.freehep.util.Value;

/**
 *
 * @author Dmitry Onoprienko
 * @version $Id: IntegerConditionDefinition.java 13824 2011-05-04 18:30:02Z onoprien $
 */
public class IntegerConditionDefinition extends AbstractConditionDefinition {

// -- Private parts : ----------------------------------------------------------

  int[] _range;

// -- Construction and initialization : ----------------------------------------

  public IntegerConditionDefinition(String name) {
    super(name);
  }

  public IntegerConditionDefinition(String name, String unit, String description) {
    super(name, unit, description);
  }


// -- Type : -------------------------------------------------------------------

  /** Returns the type of this condition. */
  public Type getType() {
    return Type.INTEGER;
  }


// -- Creating Conditions : ----------------------------------------------------

  /** Returns a new {@link Condition} object linked to this ConditionDefinition. */
  public IntegerCondition createCondition(boolean enabled, boolean inverted) {
    int[] range = getDomain();
    IntegerCondition c = new IntegerCondition(this, enabled, inverted, range[0], range[1]);
    _conditions.add(c);
    return c;
  }


// -- Domain : -----------------------------------------------------------------

  /**
   * Sets the domain of possible values for the condition.
   * The argument passed to this method should be a two-element int array: [min,max].
   */
  public void setDomain(Object domain) {
    _range = (int[])domain;
    fireStateChanged();
  }

  /**
   * Sets the range of possible values for the condition.
   */
  public void setDomain(int min, int max) {
    _range = new int[] {min,max};
    fireStateChanged();
  }

  /**
   * Returns the domain of possible values for the condition.
   * The returned object is a two-element array: [min,max].
   */
  public int[] getDomain() {
    return _range == null ? new int[] {0,0} : _range;
  }

  /**
   * Updates the domain to ensure it contains the specified value.
   */
  public void addToDomain(Value value) {
    addToDomain(value.getInt());
  }

  /**
   * Updates the domain to ensure it contains the specified value.
   */
  public void addToDomain(int value) {
    if (_range == null) {
      _range = new int[] {value,value};
    } else if (value < _range[0]) {
      _range[0] = value;
      fireStateChanged();
    } else if (value > _range[1]) {
      _range[1] = value;
      fireStateChanged();
    }
  }

// -----------------------------------------------------------------------------
}
