package hep.wired.cut;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Polygon;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.text.DecimalFormat;
import java.util.ArrayList;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JLayeredPane;
import javax.swing.JPanel;

/**
 *
 * @author Dmitry Onoprienko
 * @version $Id: $
 */
public class Slider extends JPanel {

// -- Private parts : ----------------------------------------------------------

  private Integer RULLER_LAYER = new Integer(1);
  private Integer TRACK_LAYER = new Integer(2);
  private Integer RANGE_LAYER = new Integer(3);
  private Integer KNOB_LAYER = new Integer(4);
  private Integer MARK_LAYER = new Integer(5);

  private SliderEditor _model;

  private final int _knobWidth = 15;
  private final int _knobHeight = 15;
  private final int _majorTickWidth = 10;
  private final int _minorTickWidth = 5;
  private final int _gap1 = 3;
  private final int _gap2 = 3;
  private final int _trackWidth = 5;

  private int _rullerWidth;
  private int _markSize;

  private JLayeredPane _layeredPane;
  private Ruler _ruller;

  private JButton _highKnob;
  private JButton _lowKnob;
  private JPanel _highMark;
  private JPanel _lowMark;
  private JPanel _track;
  private JPanel _range;

  int _size;  // length of the accessible slider (height of layered pane - 2 * _knobHeight)
  int _low; // current position of low knob
  int _high; // current position of high knob

  private boolean _lowMoved, _highMoved; // true if knob has been moved since the last time mouse button was released

// -- Construction and initialization : ----------------------------------------

  public Slider() {
    this(null);
  }

  public Slider(SliderEditor model) {

    setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
    setBorder(BorderFactory.createEmptyBorder(4,4,4,4));

    _layeredPane = new JLayeredPane();
    _layeredPane.setMinimumSize(new Dimension(_knobWidth+_gap1+_majorTickWidth+_gap2, 2*_knobHeight+20));
    _layeredPane.setPreferredSize(new Dimension(_knobWidth+_gap1+_majorTickWidth+_gap2, 20000));
    add(_layeredPane);

    _track = new JPanel();
    _track.setBorder(BorderFactory.createEtchedBorder());
    _layeredPane.add(_track, TRACK_LAYER);

    _range = new JPanel();
    _range.setBorder(BorderFactory.createEtchedBorder());
    _range.setBackground(Color.LIGHT_GRAY);
    _range.setOpaque(true);
    _layeredPane.add(_range, RANGE_LAYER);

    _markSize = _majorTickWidth-_minorTickWidth + _gap1;
    _highMark = new JPanel() {
      Polygon p = new Polygon(new int[]{0,0,_markSize}, new int[]{_markSize,0,_markSize}, 3);
      protected void paintComponent(Graphics g) {
        super.paintComponent(g);
        g.setColor(Color.DARK_GRAY);
        g.fillPolygon(p);
      }
    };
    Dimension mark = new Dimension(_markSize, _markSize);
    _highMark.setPreferredSize(mark);
    _highMark.setMinimumSize(mark);
    _highMark.setMaximumSize(mark);
    _highMark.setOpaque(false);
    _layeredPane.add(_highMark, MARK_LAYER);
    _lowMark = new JPanel() {
      Polygon p = new Polygon(new int[]{0,0,_markSize}, new int[]{_markSize,0,0}, 3);
      protected void paintComponent(Graphics g) {
        super.paintComponent(g);
        g.setColor(Color.DARK_GRAY);
        g.fillPolygon(p);
      }
    };
    _lowMark.setPreferredSize(mark);
    _lowMark.setMinimumSize(mark);
    _lowMark.setMaximumSize(mark);
    _lowMark.setOpaque(false);
    _layeredPane.add(_lowMark, MARK_LAYER);


    Dimension knob = new Dimension(_knobWidth, _knobHeight);
    _highKnob = new JButton();
    _highKnob.setPreferredSize(knob);
    _highKnob.setMinimumSize(knob);
    _highKnob.setMaximumSize(knob);
    _layeredPane.add(_highKnob, KNOB_LAYER);
    _lowKnob = new JButton();
    _lowKnob.setPreferredSize(knob);
    _lowKnob.setMinimumSize(knob);
    _lowKnob.setMaximumSize(knob);
    _layeredPane.add(_lowKnob, KNOB_LAYER);

    KnobMouseListener knobListener = new KnobMouseListener() {
      public void mousePressed(MouseEvent e) {
        super.mousePressed(e);
        _lowMoved = true;
      }
      public void mouseDragged(MouseEvent e) {
        _low +=  e.getY() - y;
        if (_low > _size) {
          _low = _size;
        } else if (_low < 0) {
          _low = 0;
        }
        _lowKnob.setLocation(0, _low + _knobHeight);
        if (_low < _high) {
          _highMoved = true;
          _high = _low;
          _highKnob.setLocation(0, _high);
          _model.updateFromSlider(_low, _high, true);
        } else {
          _model.updateFromSliderLow(_low, true);
        }
        _range.setBounds((_knobWidth-_trackWidth)/2-1, _high+_knobHeight/2, _trackWidth+2, _low - _high +_knobHeight);
        
      }
    };
    _lowKnob.addMouseListener(knobListener);
    _lowKnob.addMouseMotionListener(knobListener);

    knobListener = new KnobMouseListener() {
      public void mousePressed(MouseEvent e) {
        super.mousePressed(e);
        _highMoved = true;
      }
      public void mouseDragged(MouseEvent e) {
        _high +=  e.getY() - y;
        if (_high < 0) {
          _high = 0;
        } else if (_high > _size) {
          _high = _size;
        }
        _highKnob.setLocation(0, _high);
        if (_high > _low) {
          _lowMoved = true;
          _low = _high;
          _lowKnob.setLocation(0, _low + _knobHeight);
          _model.updateFromSlider(_low, _high, true);
        } else {
          _model.updateFromSliderHigh(_high, true);
        }
        _range.setBounds((_knobWidth-_trackWidth)/2-1, _high+_knobHeight/2, _trackWidth+2, _low - _high +_knobHeight);
        
      }
    };
    _highKnob.addMouseListener(knobListener);
    _highKnob.addMouseMotionListener(knobListener);

    setModel(model);
  }

  public void doLayout() {

    super.doLayout();
    if (_lowMoved || _highMoved) return;
    _size = _layeredPane.getHeight() - 2 * _knobHeight;

    _track.setPreferredSize(new Dimension(5, _layeredPane.getHeight()));
    _track.setSize(new Dimension(_trackWidth, _layeredPane.getHeight()));
    _track.setLocation((_knobWidth-_trackWidth)/2, 0);

    if (_model != null) {
      _model.setScale();
      createRuler();
      updateKnobPositions(_model.getLow(), _model.getHigh());
      updateMarkPositions(_model.getLow(), _model.getHigh());
    }
//    System.out.println("1 pref:"+ _layeredPane.getPreferredSize());
//    System.out.println("1 min:"+ _layeredPane.getMinimumSize());
//    System.out.println("1 act:"+ _layeredPane.getSize());
    revalidate();
//    System.out.println("2 pref:"+ _layeredPane.getPreferredSize());
//    System.out.println("2 min:"+ _layeredPane.getMinimumSize());
//    System.out.println("2 act:"+ _layeredPane.getSize());
  }

// -- Setters/Updaters : -------------------------------------------------------

  public void setModel(SliderEditor model) {
    if (model == null) {
      _model = null;
      _size = _layeredPane.getHeight();
      updateKnobPositions(_size - _knobHeight, _knobHeight);
      setEnabled(false);
    } else {
      _model = model;
      _highKnob.setBounds(0, _model.getHigh(), _knobWidth, _knobHeight);
      _lowKnob.setBounds(0, _model.getLow() + _knobHeight, _knobWidth, _knobHeight);
      _highMark.setBounds(_knobWidth, 0, _markSize, _markSize);
      _lowMark.setBounds(_knobWidth, 0, _markSize, _markSize);
      setEnabled(true);
      doLayout();
    }
    _lowMoved = false;
    _highMoved = false;
  }

  void updateKnobPositions(int low, int high) {
    _low = low;
    _high = high;
    _lowKnob.setLocation(0, _low + _knobHeight);
    _highKnob.setLocation(0, _high);
    _range.setBounds((_knobWidth-_trackWidth)/2-1, _high+_knobHeight/2, _trackWidth+2, _low - _high +_knobHeight);
  }

  void updateKnobPositionLow(int low) {
    _low = low;
    _lowKnob.setLocation(0, _low + _knobHeight);
    _range.setBounds((_knobWidth-_trackWidth)/2-1, _high+_knobHeight/2, _trackWidth+2, _low - _high +_knobHeight);
  }

  void updateKnobPositionHigh(int high) {
    _high = high;
    _highKnob.setLocation(0, _high);
    _range.setBounds((_knobWidth-_trackWidth)/2-1, _high+_knobHeight/2, _trackWidth+2, _low - _high +_knobHeight);
  }

  void updateMarkPositions(int low, int high) {
    _highMark.setLocation(_knobWidth, high+1 + _knobHeight - _markSize);
    _lowMark.setLocation(_knobWidth, low + _knobHeight);
  }

  void updateMarkPositionLow(int low) {
    _lowMark.setLocation(_knobWidth, low + _knobHeight);
  }

  void updateMarkPositionHigh(int high) {
    _highMark.setLocation(_knobWidth, high+1 + _knobHeight - _markSize);
  }
 

// -- Painting ruller : --------------------------------------------------------

  /** Taking into account current size of the slider, paints ticks and labels. */
  private void createRuler() {
    _ruller = _model.createRuler();
    _rullerWidth = _majorTickWidth + _gap2 + _ruller.maxLabelWidth;
    _layeredPane.setMinimumSize(new Dimension(_knobWidth + _gap1 + _rullerWidth, 20 + 2*_knobHeight));
    _layeredPane.setPreferredSize(new Dimension(_knobWidth + _gap1 + _rullerWidth, 2000));
    for (Component c : _layeredPane.getComponentsInLayer(RULLER_LAYER)) _layeredPane.remove(c);
    JPanel panel = new JPanel() {
      protected void paintComponent(Graphics g) {
        super.paintComponent(g);
        paintRuller(g);
      }
    };
    panel.setLayout(null);
    _layeredPane.add(panel, RULLER_LAYER);
    panel.setBounds(_knobWidth + _gap1, 0, _rullerWidth, _size + 2*_knobHeight);
  }

  private void paintRuller(Graphics g) {
    Font font = getFont();
    FontMetrics fm = getFontMetrics(font);
    int offset = _knobHeight + font.getSize()/2;
    for (int i=0; i < _ruller.labels.size(); i++) {
      String label = _ruller.labels.get(i);
      g.drawString(label, _rullerWidth - fm.stringWidth(label), _ruller.labelPos.get(i) +  offset);
    }
    g.setColor(Color.GRAY);
    for (int y : _ruller.major) {
      y += _knobHeight;
      g.drawLine(0, y, _majorTickWidth, y);
    }
    offset = _majorTickWidth - _minorTickWidth;
    for (int y : _ruller.minor) {
      y += _knobHeight;
      g.drawLine(offset, y, _majorTickWidth, y);
    }
  }

  public String format(double x) {
    DecimalFormat _format1 = new DecimalFormat("0.0####E0");
    DecimalFormat _format2 = new DecimalFormat("###0.0##");
    DecimalFormat _format3 = new DecimalFormat("0.0####E0");
    if (x >= 10000.) {
      return _format1.format(x);
    } else if (x >= 0.001) {
      return _format2.format(x);
    } else {
      return _format3.format(x);
    }
  }

// -- Mouse listener class : ---------------------------------------------------

  private class KnobMouseListener extends MouseAdapter {
    protected int y;
    public void mousePressed(MouseEvent e) {
      y = e.getY();
    }
    public void mouseReleased(MouseEvent e) {
      if (_model == null) return;
      if (_lowMoved) {
        if (_highMoved) {
          _model.updateFromSlider(_low, _high, false);
        } else {
          _model.updateFromSliderLow(_low, false);
        }
      } else if (_highMoved) {
        _model.updateFromSliderHigh(_high, false);
      }
      _lowMoved = false;
      _highMoved = false;
    }
  }

// -- Ruler class : -----------------------------------------------------------

  Ruler getNewRuler() {
    return new Ruler(12);
  }

  Ruler getNewRuler(int n) {
    return new Ruler(n);
  }

  class Ruler {
    ArrayList<Integer> labelPos;
    ArrayList<String> labels;
    ArrayList<Integer> major;
    ArrayList<Integer> minor;
    int maxLabelWidth;
    FontMetrics fm;
    int nDiv;
    Ruler() {this(0);}
    Ruler(int nLabels) {
      nDiv = Math.min(10, _size/_layeredPane.getFont().getSize()/4);
      if (nDiv < 1) nDiv = 1;
      if (nLabels < 1) nLabels = nDiv + 2;
      labelPos = new ArrayList(nLabels);
      labels = new ArrayList(nLabels);
      major = new ArrayList(3*nLabels);
      minor = new ArrayList(11*nLabels);
      fm = _layeredPane.getFontMetrics(_layeredPane.getFont());
      maxLabelWidth = 0;
    }
    void addLabel(int pos, String label) {
      labelPos.add(pos);
      labels.add(label);
      maxLabelWidth = Math.max(maxLabelWidth, fm.stringWidth(label));
    }
    void addMajorTick(int pos) {
      major.add(pos);
    }
    void addMinorTick(int pos) {
      minor.add(pos);
    }
  }

// -----------------------------------------------------------------------------
}
