// Copyright 2004, FreeHEP.
package hep.wired.edit;

import hep.wired.feature.Resetable;
import hep.wired.services.GraphicsPanel;
import hep.wired.services.RecordPlot;

import java.util.logging.Logger;

import javax.swing.undo.UndoableEdit;

/**
 * Resets the plot to its initial state.
 * 
 * @author Mark Donszelmann
 * @version $Id: Reset.java 8598 2006-08-14 20:38:02Z duns $
 */
public class Reset extends WiredEdit implements GraphicsPanelEdit {

    private static Logger logger = Logger.getLogger(Reset.class.getPackage()
            .getName());

    private Object oldState;

    /**
     * Creates a Reset edit.
     */
    public Reset() {
        super();
        oldState = null;
    }

    public WiredEdit copy(RecordPlot plot) {
        WiredEdit copy = new Reset();
        copy.setRecordPlot(plot);

        return copy;
    }

    public String getPresentationName() {
        return "Reset";
    }

    public boolean addEdit(UndoableEdit edit) {
        if (edit instanceof Reset) {
            // absorb
            return true;
        }
        return super.addEdit(edit);
    }

    public boolean isSupportedBy(GraphicsPanel p) {
        return (p.getFeature(Resetable.class) != null);
    }

    protected void redoEdit() {
        Resetable resetable = (Resetable) getRecordPlot().getGraphicsPanel()
                .getFeature(Resetable.class);
        if (resetable != null) {
            oldState = resetable.reset(oldState);
            getRecordPlot().repaint();
        }
        logger.finer(toString());
    }

    protected void undoEdit() {
        Resetable resetable = (Resetable) getRecordPlot().getGraphicsPanel()
                .getFeature(Resetable.class);
        if (resetable != null) {
            oldState = resetable.reset(oldState);
            getRecordPlot().repaint();
        }
        logger.finer(toString());
    }

}
