// Copyright 2004-2005, FreeHEP.
package hep.wired.edit;

import hep.wired.feature.Translateable2D;
import hep.wired.feature.Translateable3D;
import hep.wired.services.GraphicsPanel;
import hep.wired.services.RecordPlot;
import hep.wired.util.XYZindices;

import java.util.logging.Logger;

import javax.swing.undo.UndoableEdit;

/**
 * Set Translate a plot in 2D or 3D.
 * 
 * @author Mark Donszelmann
 * @version $Id: SetTranslate.java 8598 2006-08-14 20:38:02Z duns $
 */

public class SetTranslate extends WiredEdit implements GraphicsPanelEdit,
        XYZindices {

    private static Logger logger = Logger.getLogger(SetTranslate.class
            .getPackage().getName());

    private double tx, ty, tz;

    private double[] oldT;

    /**
     * Creates a translate edit with given translation in model units.
     */
    public SetTranslate(double tx, double ty, double tz) {
        super(0);
        this.tx = tx;
        this.ty = ty;
        this.tz = tz;
        oldT = null;
    }

    public WiredEdit copy(RecordPlot plot) {
        WiredEdit copy = new SetTranslate(tx, ty, tz);
        copy.setRecordPlot(plot);
        return copy;
    }

    public String getPresentationName() {
        StringBuffer s = new StringBuffer();
        s.append(toString());
        s.append(" (");
        s.append(format.format(tx));
        s.append(", ");
        s.append(format.format(ty));
        if (tz != 0) {
            s.append(", ");
            s.append(format.format(tz));
        }
        s.append(")");
        return s.toString();
    }

    public String toString() {
        return "SetTranslate";
    }

    public boolean addEdit(UndoableEdit edit) {
        return false;
    }

    public boolean isSupportedBy(GraphicsPanel p) {
        return p.supports(Translateable2D.class)
                || p.supports(Translateable3D.class);
    }

    protected void redoEdit() {
        GraphicsPanel panel = getRecordPlot().getGraphicsPanel();
        Translateable3D translateable3D = (Translateable3D) panel
                .getFeature(Translateable3D.class);
        if (translateable3D != null) {
            oldT = translateable3D.getTranslate();
            translateable3D.setTranslate(tx, ty, tz);
        } else {
            Translateable2D translateable2D = (Translateable2D) panel
                    .getFeature(Translateable2D.class);
            if (translateable2D == null) {
                System.err.println("Could not redo: " + this);
                return;
            }
            oldT = translateable2D.getTranslate();
            translateable2D.setTranslate(tx, ty);
        }
        getRecordPlot().repaint();
        logger.finer(toString());
    }

    protected void undoEdit() {
        if (oldT == null) {
            System.err.println("Could not undo: " + this);
            return;
        }

        GraphicsPanel panel = getRecordPlot().getGraphicsPanel();
        Translateable3D translateable3D = (Translateable3D) panel
                .getFeature(Translateable3D.class);
        if (translateable3D != null) {
            translateable3D.setTranslate(oldT[X], oldT[Y], oldT[Z]);
        } else {
            Translateable2D translateable2D = (Translateable2D) panel
                    .getFeature(Translateable2D.class);
            if (translateable2D == null) {
                System.err.println("Could not undo: " + this);
                return;
            }
            translateable2D.setTranslate(oldT[X], oldT[Y]);
        }
        getRecordPlot().repaint();
        logger.finer(toString());
    }
}
