// Copyright 2004-2005, FreeHEP.
package hep.wired.edit;

import hep.wired.services.Edit;
import hep.wired.services.RecordPlot;

import org.freehep.swing.undo.AnimatedCompoundEdit;
import org.freehep.swing.undo.LinkableEdit;

/**
 * Class for a compound edit. A compound edit contains a series of edits (which
 * may be compound edits as well). This class keeps a reference to the plot when
 * the edit is posted to the plot.
 * 
 * @author Mark Donszelmann
 * @version $Id: WiredCompoundEdit.java 8598 2006-08-14 20:38:02Z duns $
 */
public class WiredCompoundEdit extends AnimatedCompoundEdit implements Edit {

    private RecordPlot recordPlot;

    // FIXME WIRED-407, should be accessible in AnimatedCompoundEdit.
    private String name;

    /**
     * Creates an empty composite edit with given name.
     */
    public WiredCompoundEdit(String name) {
        super(name);
        this.name = name;
    }

    public String getID() {
        return getPresentationName();
    }

    /**
     * Returns the record plot to which this edit was sent. Returns null if the
     * edit was not (yet) sent to a plot.
     */
    protected RecordPlot getRecordPlot() {
        return recordPlot;
    }

    /**
     * To be called as part of the copy procedure to set the recordPlot
     * variable.
     */
    public void setRecordPlot(RecordPlot recordPlot) {
        this.recordPlot = recordPlot;
    }

    /**
     * Creates a copy of the compound edit and its contained edits. This copy
     * method calls setRecordPlot() on each of them to initialize the recordPlot
     * variable(s).
     */
    public WiredCompoundEdit copy(RecordPlot plot) {

        WiredCompoundEdit copy = new WiredCompoundEdit(getPresentationName());
        copy.setRecordPlot(plot);

        // run over all subEdits
        LinkableEdit edit = getFirstEdit();
        while (edit != null) {
            if (edit instanceof WiredEdit) {
                copy.addEdit(((WiredEdit) edit).copy(plot));
            } else if (edit instanceof WiredCompoundEdit) {
                copy.addEdit(((WiredCompoundEdit) edit).copy(plot));
            } else {
                copy.addEdit(edit);
            }
            edit = edit.getNextEdit();
        }
        copy.end();
        return copy;
    }

    public String toString() {
        if (name != null)
            return name;
        LinkableEdit last = getLastEdit();
        if (last != null)
            return last.toString();
        return super.toString();
    }
}
