// Copyright 2004-2005, FreeHEP.
package hep.wired.edit;

import hep.wired.services.Edit;
import hep.wired.services.RecordPlot;

import org.freehep.swing.undo.AnimatedEdit;
import org.freehep.util.ScientificFormat;

/**
 * Base class for a non-animated single edit. This class keeps the plot when the
 * edit is posted to the plot.
 * 
 * @author Mark Donszelmann
 * @version $Id: WiredEdit.java 8598 2006-08-14 20:38:02Z duns $
 */
public abstract class WiredEdit extends AnimatedEdit implements Edit {

    protected static ScientificFormat format = new ScientificFormat(2, 8, false);

    private RecordPlot recordPlot;

    /**
     * Creates an Edit with zero frames.
     */
    public WiredEdit() {
        this(0);
    }

    /**
     * Creates and edit with given frames.
     */
    public WiredEdit(int frames) {
        super(frames);
    }

    public String toString() {
        return getPresentationName();
    }

    public String getID() {
        String[] names = getClass().getName().split(".");
        return names[names.length - 1];
    }

    /**
     * Returns the record plot to which this edit was sent. Returns null if the
     * edit was not (yet) sent to a plot.
     */
    protected RecordPlot getRecordPlot() {
        return recordPlot;
    }

    /**
     * To be called as part of the copy procedure to set the recordPlot
     * variable.
     */
    public void setRecordPlot(RecordPlot recordPlot) {
        this.recordPlot = recordPlot;
    }

    /**
     * Creates a copy of the edit which is attached to the plot, and thus can be
     * (re/un)done.
     */
    public abstract WiredEdit copy(RecordPlot plot);

    // Should be imlemented by all ProjectionEdits
    // public abstract Shape createTransformedShape(Component component, Shape
    // shape);

    /**
     * Called when an animation is started. Redo is set when the animation is to
     * run forward, e.g. the edit is (re-)done. Default implementation is a null
     * method.
     */
    protected void startAnimation(boolean redo) {
    };

    /**
     * Called when the given frameNo is to be shown. Default implementation is a
     * null method.
     */
    protected void showAnimation(int frameNo) {
    };

    /**
     * Called when the animation is to be ended. Default implementation is a
     * null method.
     */
    protected void endAnimation() {
    };

}
