// Copyright 2004, FreeHEP.
package hep.wired.feature;

import hep.wired.services.Feature;
import hep.wired.services.ViewPort;

/**
 * Can calculate translations from screen and model.
 * 
 * @author Mark Donszelmann
 * @version $Id: Translateable.java 8598 2006-08-14 20:38:02Z duns $
 */

public interface Translateable extends Feature {

    /**
     * Returns Model translation from screen translation. An array of length 3:
     * xyz[] = model(uvw[]). For 2D systems the 3rd coordinate may be ignored.
     * The returned array may have a lifetime up to the next call to this method
     * on this object.
     */
    public double[] getModelTranslation(double[] uvw, ViewPort viewPort);

    /**
     * Returns Screen translation from model translation. An array of length 3:
     * uvw[] = model(xyz[]). For 2D systems the 3rd coordinate may be ignored.
     * The returned array may have a lifetime up to the next call to this method
     * on this object.
     */
    public double[] getScreenTranslation(double[] xyz, ViewPort viewPort);
}
