// Copyright 2003-2006, FreeHEP.
package hep.wired.interaction;

import hep.wired.services.InteractionHandler;
import hep.wired.services.RecordPlot;

import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseWheelEvent;

import javax.swing.Icon;

import org.freehep.application.Application;
import org.freehep.xml.io.XMLIOManager;

/**
 * Near complete (null) implementation of an InteractionHandler.
 * 
 * @author Mark Donszelmann
 * @version $Id: AbstractInteractionHandler.java 13850 2011-07-06 19:23:23Z onoprien $
 */
public abstract class AbstractInteractionHandler implements InteractionHandler {

  private transient int width;
  private transient int height;
  private String name;

  /**
   * Creates an AbstractInteractionHandler.
   */
  protected AbstractInteractionHandler(String name) {
    this.name = name;
  }

  public String getID() {
    String[] names = getClass().getName().split(".");
    return names[names.length - 1];
  }

  public String getName() {
    return name;
  }

  public void setRecord(RecordPlot plot, Object record) {
  }

  public void setSelected(RecordPlot plot, boolean selected) {
//      System.out.println("selected "+ this.getName() +"  "+ selected);
  }

  public void setSize(RecordPlot plot, int width, int height) {
    this.width = width;
    this.height = height;
  }

  public int getWidth() {
    return width;
  }

  public int getHeight() {
    return height;
  }

  public Icon getIcon(int size) {
    return null;
  }

  public void reset(RecordPlot plot, InputEvent event) {
    Application.getApplication().setStatusMessage("");
  }

  public void mouseButton1Clicked(RecordPlot plot, MouseEvent event) {
  }

  public void mouseButton1Pressed(RecordPlot plot, MouseEvent event) {
  }

  public void mouseButton1Released(RecordPlot plot, MouseEvent event) {
  }

  public void mouseButton1DragStarted(RecordPlot plot, MouseEvent event) {
  }

  public void mouseButton1Dragged(RecordPlot plot, MouseEvent event) {
  }

  public void mouseButton1DragEnded(RecordPlot plot, MouseEvent event) {
  }

  public void mouseMoved(RecordPlot plot, MouseEvent event) {
//      System.out.println("y = "+ event.getY());
  }

  public void mouseEntered(RecordPlot plot, MouseEvent event) {
  }

  public void mouseExited(RecordPlot plot, MouseEvent event) {
  }

  public void mouseWheelMoved(RecordPlot plot, MouseWheelEvent event) {
  }

  public boolean escKeyPressed(RecordPlot plot, KeyEvent event) {
    return false;
  }

  public boolean escKeyReleased(RecordPlot plot, KeyEvent event) {
    return false;
  }

  public boolean altKeyPressed(RecordPlot plot, KeyEvent event) {
    return false;
  }

  public boolean altKeyReleased(RecordPlot plot, KeyEvent event) {
    return false;
  }

  public boolean ctrlKeyPressed(RecordPlot plot, KeyEvent event) {
    return false;
  }

  public boolean ctrlKeyReleased(RecordPlot plot, KeyEvent event) {
    return false;
  }

  public boolean shiftKeyPressed(RecordPlot plot, KeyEvent event) {
    return false;
  }

  public boolean shiftKeyReleased(RecordPlot plot, KeyEvent event) {
    return false;
  }

  public boolean metaKeyPressed(RecordPlot plot, KeyEvent event) {
    return false;
  }

  public boolean metaKeyReleased(RecordPlot plot, KeyEvent event) {
    return false;
  }

  public boolean ctrlMetaKeyPressed(RecordPlot plot, KeyEvent event) {
    return false;
  }

  public boolean ctrlMetaKeyReleased(RecordPlot plot, KeyEvent event) {
    return false;
  }

  public boolean otherKeyPressed(RecordPlot plot, KeyEvent event) {
    return false;
  }

  public boolean otherKeyReleased(RecordPlot plot, KeyEvent event) {
    return false;
  }

  public boolean keyTyped(RecordPlot plot, KeyEvent event) {
    return false;
  }

  //
  // XMLIO
  //
  public void save(XMLIOManager xmlioManager, org.jdom.Element nodeEl) {
    // just save class
  }

  public void restore(XMLIOManager xmlioManager, org.jdom.Element nodeEl) {
    // just restore class
  }

// -- Equality : ---------------------------------------------------------------
  /** Overridden to use interaction handler name in comparisons. */
  public boolean equals(Object other) {
    if (other instanceof InteractionHandler) {
      return name.equals(((InteractionHandler) other).getName());
    } else {
      return false;
    }
  }

  /** Overridden to use interaction handler name to generate hash code. */
  public int hashCode() {
    return name.hashCode();
  }
}
