// Copyright 2004-2006, FreeHEP.
package hep.wired.interaction;

import hep.wired.edit.Rotate;
import hep.wired.feature.Rotateable3D;
import hep.wired.image.WiredBaseImage;
import hep.wired.services.GraphicsPanel;
import hep.wired.services.RecordPlot;

import java.awt.event.InputEvent;
import java.awt.event.MouseEvent;

import javax.swing.Icon;

import org.freehep.application.Application;

/**
 * Drag mouse to rotate plot using a virtual trackball.
 * 
 * @author Mark Donszelmann
 * @version $Id: DragBallToRotate.java 8598 2006-08-14 20:38:02Z duns $
 */
public class DragBallToRotate extends DefaultInteractionHandler {

    private int xp, yp;

    private double ballRadius = 100;

    /**
     * Create a ball handler to generate rotate edits.
     */
    public DragBallToRotate() {
        super("Drag Ball to Rotate");
    }

    public Icon getIcon(int size) {
        return WiredBaseImage.getIcon("RotateBall%w", size);
    }

    public String getDescription() {
        return "Drag to rotate using virtual ball; Shift-drag to rotate over vertical axis; "+(isMac() ? command : "Ctrl")+"-drag to rotate over horizontal axis.";
    }

    public boolean isSupportedBy(GraphicsPanel panel) {
        return (new Rotate().isSupportedBy(panel))
                && panel.supports(Rotateable3D.class);
    }

    public void changeCursor(RecordPlot plot, InputEvent event) {
        plot.setCursor(WiredBaseImage.getBestCursor("RotateBallCursor%w", 32,
                32));
    }

    public void reset(RecordPlot plot, InputEvent event) {
        Application.getApplication().setStatusMessage(getDescription());
    }

    public void mouseEntered(RecordPlot plot, MouseEvent event) {
        plot.requestFocusInWindow();
        changeCursor(plot, event);
    }

    public void mouseButton1DragStarted(RecordPlot plot, MouseEvent event) {
        xp = event.getX();
        yp = event.getY();
        plot.beginUpdate();
        plot.getGraphicsPanel().setFastMode(true);
    }

    public void mouseButton1Dragged(RecordPlot plot, MouseEvent event) {
        int xd = event.getX();
        int yd = event.getY();

        // mimic ball rotation,
        // see "Andrew J. Hanson, The Rolling Ball, Graphics Gems III [KIRK,
        // 1992], pp. 51-60."

        // vector in right-hand screen coordinate system
        int dx = isControlMetaDown(event) ? 0 : xd - xp;
        int dy = event.isShiftDown() ? 0 : -(yd - yp);

        // input device displacement
        double drSquare = dx * dx + dy * dy;
        double dr = Math.sqrt(drSquare);

        // bail out if there is no displacement
        if (dr == 0)
            return;

        // unit vector in screen plane
        // NOTE, remove minus to get anti-ball rotation effect for small circle
        // dragging
        double nx = -dy / dr;
        double ny = dx / dr;
        double nz = 0;

        // ball radius
        double R = ballRadius;

        // theta
        double cosTheta = R / Math.sqrt(R * R + drSquare);
        double theta = Math.acos(cosTheta);

        plot.postEdit(new Rotate(theta, nx, ny, nz));

        xp = xd;
        yp = yd;
    }

    public void mouseButton1DragEnded(RecordPlot plot, MouseEvent event) {
        plot.getGraphicsPanel().setFastMode(false);
        plot.endUpdate();
        plot.repaint();
    }

    public String toString() {
        return "Drag to Rotate in 3D";
    }
}
