// Copyright 2004, FreeHEP.
package hep.wired.plugin;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;

import org.freehep.xml.io.XMLIOFactory;

/**
 * Factory to create objects by calling the default constructor.
 * 
 * Should maybe move to FreeHEP (xml.io)
 * 
 * @author Mark Donszelmann
 * @version $Id: AbstractXMLIOFactory.java 8598 2006-08-14 20:38:02Z duns $
 */
public abstract class AbstractXMLIOFactory implements XMLIOFactory {

    public Object createObject(Class cls) {
        try {
            Constructor ctr = cls.getDeclaredConstructor(null);
            ctr.setAccessible(true);
            return ctr.newInstance(null);
        } catch (NoSuchMethodException e) {
            throw new IllegalArgumentException(cls.getName()
                    + " does not have no-arg constructor.");
        } catch (InvocationTargetException e) {
            throw new IllegalArgumentException(cls.getName()
                    + " cannot call no-arg constructor.");
        } catch (InstantiationException e) {
            throw new IllegalArgumentException(cls.getName()
                    + " cannot be instantiated.");
        } catch (IllegalAccessException e) {
            throw new IllegalArgumentException(cls.getName()
                    + " cannot be instantiated due to access problems.");
        }
    }
}
