// Copyright 2003-2005, FreeHEP.
package hep.wired.plugin;

import hep.wired.plot.WiredPlot;
import hep.wired.services.InteractionHandler;
import hep.wired.services.ViewPort;
import hep.wired.util.Matrix2D;
import hep.wired.util.Matrix3D;
import hep.wired.util.WiredRegistry;

import java.io.IOException;
import java.net.URL;
import java.util.Iterator;
import java.util.Set;

import org.freehep.application.studio.Plugin;
import org.freehep.application.studio.Studio;
import org.freehep.xml.io.XMLIORegistry;
import org.freehep.xml.menus.XMLMenuBuilder;
import org.xml.sax.SAXException;

/**
 * Defines the Library plugin for use in JAS3.
 * 
 * @author Mark Donszelmann
 * @version $Id: WiredBase.java 13845 2011-06-09 00:14:38Z onoprien $
 */
public class WiredBase extends Plugin {

    // See below
    // private WiredBaseCommandHandler baseHandler;

    public WiredBase() {
        // See below
        // baseHandler = new WiredBaseCommandHandler(this);
    }

    protected void init() throws SAXException, IOException {
        Studio application = getApplication();
        WiredRegistry.add(this);

        // user = application.getUserProperties();

        // NOTE, should just be done in save/restore but that seems too late.
        addXMLIOFactories(null);

        XMLMenuBuilder builder = application.getXMLMenuBuilder();
        URL xml = getClass().getResource("WiredBase.menus");
        try {
            builder.build(xml);
        } catch (org.xml.sax.SAXParseException e) {
            System.out.println(e);
            System.out.println("at line: " + e.getLineNumber() + " column: "
                    + e.getColumnNumber());
            throw e;
        }

        application.addToolBar(builder.getToolBar("interactionToolBar"), "Interaction Toolbar");
        application.addToolBar(builder.getToolBar("actionToolBar"), "Action Toolbar");

        // Do not enable, since WIRED4 needs to handle commands differently. For
        // JSky add the handler in the app itself.
        // application.getCommandTargetManager().add(baseHandler);
    }

    // FIXME WIRED-391 maybe move to registry...
    private void addXMLIOFactories(XMLIORegistry registry) {
        addXMLIOFactory(registry, WiredPage.class);
        addXMLIOFactory(registry, WiredPlot.class);

        addXMLIOFactory(registry, Matrix3D.class);
        addXMLIOFactory(registry, Matrix2D.class);

        addXMLIOFactoryForService(registry, InteractionHandler.class);
        addXMLIOFactoryForService(registry, ViewPort.class);
    }

    // FIXME WIRED-391
    private void addXMLIOFactory(XMLIORegistry registry, Class cls) {
        if ((registry == null) || (registry.getXMLIOFactory(cls) == null)) {
            WiredRegistry.add(new WiredXMLIOFactory(cls));
        }
    }

    private void addXMLIOFactoryForService(XMLIORegistry registry, Class service) {
        Set allClasses = WiredRegistry.allClasses(service);
        for (Iterator i = allClasses.iterator(); i.hasNext();) {
            addXMLIOFactory(registry, (Class) i.next());
        }
    }
}
