// Copyright 2004-2005, FreeHEP.
package hep.wired.services;

import hep.wired.util.UVWindices;
import hep.wired.util.WiredRegistry;
import hep.wired.util.XYZindices;

import org.freehep.xml.io.XMLIO;

/**
 * Transforms coordinates in a world-unit system (X,Y,Z) with intervals (-1,1)
 * into screen coordinates (U,V,W).
 * 
 * @author Mark Donszelmann
 * @version $Id: ViewPort.java 8598 2006-08-14 20:38:02Z duns $
 */

public interface ViewPort extends WiredRegistry.ID, XYZindices, UVWindices,
        XMLIO {

    /**
     * Returns the name of the View Port.
     */
    public String getName();

    /**
     * Returns a copy of the viewport
     */
    public ViewPort copy();

    /**
     * Returns an array of length 3: uvw = viewport(xyz). The z component is not
     * touched. The returned array may have a lifetime up to the next call to
     * this method on this object.
     */
    public double[] transform(double[] xyz);

    /**
     * Returns an array of length 3*n: uvw[][n] = viewport(xyz[][n]). The z
     * component is not touched. The returned array may have a lifetime up to
     * the next call to this method on this object. The number of "valid"
     * coordinates is given by n.
     */
    public double[][] transform(double[][] xyz, int n);

    /**
     * Returns an array of length 3: uvw[] = viewport-delta(xyz[]). The z
     * component is not touched. The returned array may have a lifetime up to
     * the next call to this method on this object.
     */
    public double[] deltaTransform(double[] xyz);

    /**
     * Returns an array of length 3: xyz[] = inverse-viewport(uvw[]). The w
     * component is not touched. The returned array may have a lifetime up to
     * the next call to this method on this object.
     */
    public double[] inverseTransform(double[] uvw);

    /**
     * Returns an array of length 3: xyz[] = inverse-viewport-delta(uvw[]). The
     * w component is not touched. The returned array may have a lifetime up to
     * the next call to this method on this object.
     */
    public double[] inverseDeltaTransform(double[] uvw);
}
