// Copyright 2003-2005, FreeHEP.
package hep.wired.util;

import java.awt.Color;
import java.awt.Component;
import java.text.Format;
import java.util.Vector;

import javax.swing.JComponent;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.event.TableModelEvent;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumnModel;
import javax.swing.table.TableModel;

import org.freehep.swing.table.TableColumnPacker;
import org.freehep.swing.table.TableSorter;
import org.freehep.util.ScientificFormat;

/**
 * Improved table supporting, variable width columns, multi-type cells per
 * column, Color as a celltype, sorting on columns, tooltips on cells, etc.
 * 
 * @author Mark Donszelmann
 * @version $Id: ScientificTable.java 8598 2006-08-14 20:38:02Z duns $
 */
public class ScientificTable extends JTable {

    private TableColumnPacker packer = new TableColumnPacker();

    private ScientificFormat format = new ScientificFormat();

    public ScientificTable() {
        super();
        init();
    }

    public ScientificTable(int rows, int columns) {
        super(rows, columns);
        init();
    }

    public ScientificTable(Object[][] rowData, Object[] columnNames) {
        super(rowData, columnNames);
        init();
    }

    public ScientificTable(TableModel dm) {
        super(dm);
        init();
    }

    public ScientificTable(TableModel dm, TableColumnModel cm) {
        super(dm, cm);
        init();
    }

    public ScientificTable(TableModel dm, TableColumnModel cm,
            ListSelectionModel sm) {
        super(dm, cm, sm);
        init();
    }

    public ScientificTable(Vector rowData, Vector columnNames) {
        super(rowData, columnNames);
        init();
    }

    private void init() {
        // Add color as a type
        setDefaultRenderer(Color.class, new DefaultTableCellRenderer() {
            protected void setValue(Object value) {
                setText(null);
                setBackground((Color) value);
            }
        });

        // Add Double as a type using scientific format
        setDefaultRenderer(Double.class, new DefaultTableCellRenderer() {
            protected void setValue(Object value) {
                setText(format.format(((Double) value).doubleValue()));
                setHorizontalAlignment(RIGHT);
            }
        });

        // add column sorter
        new TableSorter(this);

        // add packer and leave place for sorter symbol
        packer.setHeaderMargin(15);
    }

    // override to allow different type of cells in one column
    public TableCellRenderer getCellRenderer(int row, int column) {
        Object value = getValueAt(row, column);
        return (value != null) ? getDefaultRenderer(value.getClass()) : super
                .getCellRenderer(row, column);
    }

    // override to allow different type of cells in one column
    public TableCellEditor getCellEditor(int row, int column) {
        Object value = getValueAt(row, column);
        // System.err.println("Getting Editor for "+value+" of class: "+((value
        // != null) ? value.getClass() : null));
        return (value != null) ? getDefaultEditor(value.getClass()) : super
                .getCellEditor(row, column);
    }

    // override for tooltip text
    public Component prepareRenderer(TableCellRenderer renderer, int rowIndex,
            int vColIndex) {
        Component c = super.prepareRenderer(renderer, rowIndex, vColIndex);
        if (c instanceof JComponent) {
            Object v = getValueAt(rowIndex, vColIndex);
            if (v != null)
                ((JComponent) c).setToolTipText(v.toString());
        }
        return c;
    }

    public void tableChanged(TableModelEvent event) {
        super.tableChanged(event);
        if (packer != null)
            packer.packColumns(this);
    }

    public void setFormat(ScientificFormat format) {
        this.format = format;
        doLayout();
    }

    public Format getFormat() {
        return format;
    }
}
