// Copyright 2004, FreeHEP.
package hep.wired.util;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.Set;

import org.freehep.application.Application;
import org.freehep.application.studio.Studio;
import org.freehep.util.FreeHEPLookup;
import org.openide.util.Lookup;

/**
 * Utility class to lookup values in the registry.
 * 
 * @author Mark Donszelmann
 * @version $Id: WiredRegistry.java 8598 2006-08-14 20:38:02Z duns $
 */

public class WiredRegistry {

    private static transient FreeHEPLookup registry;

    private WiredRegistry() {
    }

    public static void add(Object obj) {
        if (registry == null)
            registry = ((Studio) Application.getApplication()).getLookup();
        registry.add(obj);
    }

    public static Object lookup(Class cls) {
        if (registry == null)
            registry = ((Studio) Application.getApplication()).getLookup();
        Collection instances = registry.lookup(new Lookup.Template(cls))
                .allInstances();
        return instances.isEmpty() ? null : instances.iterator().next();
    }

    public static Object lookup(Class cls, String name) {
        Collection instances = registry.lookup(new Lookup.Template(cls))
                .allInstances();
        for (Iterator i = instances.iterator(); i.hasNext();) {
            Object obj = i.next();
            if (obj instanceof ID) {
                if (((ID) obj).getID().equals(name))
                    return obj;
            }
        }
        return null;
    }

    public static Collection allInstances(Class cls) {
        if (registry == null)
            registry = ((Studio) Application.getApplication()).getLookup();
        return registry.lookup(new Lookup.Template(cls)).allInstances();
    }

    public static Collection allInstances(Class cls, String name) {
        Collection instancesWithID = new ArrayList();
        Collection instances = registry.lookup(new Lookup.Template(cls))
                .allClasses();
        for (Iterator i = instances.iterator(); i.hasNext();) {
            Object obj = i.next();
            if (obj instanceof ID) {
                if (((ID) obj).getID().equals(name))
                    instancesWithID.add(obj);
            }
        }
        return instancesWithID;
    }

    public static Set allClasses(Class cls) {
        return registry.lookup(new Lookup.Template(cls)).allClasses();
    }

    public static interface ID {
        public String getID();
    }
}
