// Copyright 2005, FreeHEP.
package hep.wired.util.slider;

import javax.swing.DefaultBoundedRangeModel;

/**
 * Like DefaultBoundedRangeModel but for integer values.
 * 
 * @author Mark Donszelmann
 * @version $Id: DefaultBoundedIntegerRangeModel.java 2156 2005-08-02 18:47:04Z
 *          duns $
 */
public class DefaultBoundedIntegerRangeModel extends DefaultBoundedRangeModel
        implements BoundedIntegerRangeModel {

    private int def;

    public DefaultBoundedIntegerRangeModel(int value, int extent, int min,
            int max) {
        super(value, extent, min, max);

        def = value;
    }

    public int getDefaultValue() {
        return def;
    }

    public void setDefaultValue(int def) {
        setRangeProperties(getValue(), getExtent(), getMinimum(), getMaximum(),
                def, getValueIsAdjusting());
    }

    public static boolean rangePropertiesChanged(BoundedIntegerRangeModel model, int newValue, int newExtent,
            int newMin, int newMax, int newDef, boolean newAdjusting) {
        boolean deltaValue = newValue != model.getValue();
        boolean deltaExtent = newExtent != model.getExtent();
        boolean deltaMin = newMin != model.getMinimum();
        boolean deltaMax = newMax != model.getMaximum();
        boolean deltaDef = newDef != model.getDefaultValue();
        boolean deltaAdjusting = newAdjusting != model.getValueIsAdjusting();
        return deltaValue || deltaExtent || deltaMin || deltaMax || deltaDef || deltaAdjusting;
    }

    public void setRangeProperties(int newValue, int newExtent, int newMin,
            int newMax, int newDef, boolean newAdjusting) {
        boolean fire = (newDef != def);

        if (fire) {
            def = newDef;
        }

        boolean isChanged = ((newValue != getValue())
                || (newExtent != getExtent()) || (newMin != getMinimum())
                || (newMax != getMaximum()) || (newAdjusting != getValueIsAdjusting()));
        if (isChanged) {
            super.setRangeProperties(newValue, newExtent, newMin, newMax,
                    newAdjusting);
        } else {
            if (fire)
                fireStateChanged();
        }
    }
}
