package hep.wired.util.tree;

import java.awt.Component;
import java.awt.Graphics;
import java.awt.Insets;
import javax.swing.ButtonModel;
import javax.swing.Icon;
import javax.swing.JCheckBox;

/**
 *
 * @author Dmitry Onoprienko
 * @version $Id: $
 */
public class BranchCheckBox extends JCheckBox {

// -- Enumeration of states (mostly for compatibility with Mark's code) : ------

  public enum State {ALL, NONE, PART};

// -- Private parts : ----------------------------------------------------------

  private boolean _all;

  private JCheckBox _dummy;
  private ButtonModel _model;

  private int i=0;

// -- Constructors : -----------------------------------------------------------

  public BranchCheckBox() {
    this(true,true);
  }

  public BranchCheckBox(boolean selected, boolean full) {
    super((Icon)null, selected);
    _all = full;
    _dummy = new JCheckBox();
    _dummy.setOpaque(true);
    super.setOpaque(true);
    setRolloverEnabled(false);
    _model = _dummy.getModel();
  }

// -- Setters : ----------------------------------------------------------------

  public void setBranchComplete(boolean complete) {
    _all = complete;
    this.repaint();
  }

  public void setState(State state) {
    if (state == State.ALL) {
      _all = true;
      setSelected(true);
    } else if (state == State.PART) {
      _all = false;
      setSelected(true);
    } else {
      setSelected(false);
    }
  }

// -- Getters : ----------------------------------------------------------------

  public boolean isBranchComplete() {
    return _all;
  }

// -- Forward painting and its setters to dummy JCheckBox : --------------------

  public void paint(Graphics g) {
    _dummy.setSelected(isSelected());
    _dummy.setEnabled(isEnabled());
    _dummy.setBounds(getBounds());
    if (isSelected()) {
      if (_all) {
        _model.setArmed(false);
        _model.setPressed(false);
//        _model.setSelected(true);
      } else {
        _model.setArmed(true);
        _model.setPressed(true);
//        _model.setSelected(true);
      }
    } else {
      _model.setArmed(false);
      _model.setPressed(false);
//      _model.setSelected(false);
    }
//    System.out.print(i++ +" Painting: ");
//    if (_model.isArmed()) System.out.print(" armed ");
//    if (_model.isPressed()) System.out.print(" pressed ");
//    if (_dummy.isSelected()) System.out.print(" selected ");
//    if (_dummy.isEnabled()) System.out.print(" enabled ");
//    System.out.println("");
    _dummy.updateUI();
    _dummy.paint(g);
  }

  public void setBorderPaintedFlat(boolean b) {
    super.setBorderPaintedFlat(b);
    _dummy.setBorderPaintedFlat(b);
  }

  public void setMargin(Insets m) {
    super.setMargin(m);
    if (_dummy != null) _dummy.setMargin(m);
  }

  public void setOpaque(boolean isOpaque) {
//    super.setOpaque(isOpaque);
//    if (_dummy != null) _dummy.setOpaque(isOpaque);
  }

  public void updateUI() {
    super.updateUI();
    if (_dummy != null) _dummy.updateUI();
  }

// -----------------------------------------------------------------------------
}
