package hep.wired.util.tree;

import java.util.Enumeration;
import javax.swing.JTree;
import javax.swing.tree.TreeCellRenderer;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;

/**
 * 
 * @author Dmitry Onoprienko
 * @version $Id: $
 */
public class WiredTree extends JTree {

  protected WiredTreeModel.MarkListener _markListener;

// -- Construction : -----------------------------------------------------------

  public WiredTree(WiredTreeModel model) {
    super(model);
    init();
  }

  public WiredTree(WiredTreeNode root) {
    this(root, false);
  }

  public WiredTree(WiredTreeNode root, boolean asksAllowsChildren) {
    super(new WiredTreeModel(root, asksAllowsChildren));
    init();
  }

  private void init() {
    setRootVisible(false);
    setShowsRootHandles(true);
    setEditable(true);
    super.setCellRenderer(new WiredTreeCellRenderer(this));
    setCellEditor(new WiredTreeCellEditor(this));
    setSelectionModel(JTree.EmptySelectionModel.sharedInstance());
  }

// -- Ops : --------------------------------------------------------------------

  public void setModel(TreeModel model) {
    setModel((WiredTreeModel)model, true);
  }

  public void setModel(WiredTreeModel model, boolean restoreExpansionState) {
    if (treeModel != null) {
      ((WiredTreeModel)treeModel).removeMarkListener(_markListener);
      removeTreeExpansionListener((WiredTreeModel)treeModel);
    }
    super.setModel(model);
    if (model != null) {
      if (restoreExpansionState) restoreExpansionState();
      if (_markListener == null) {
        _markListener = new WiredTreeModel.MarkListener() {

          public void markChanged(WiredTreeModel.MarkEvent e) {
            repaint();
          }
        };
      }
      model.addMarkListener(_markListener);
      addTreeExpansionListener(model);
    }
  }

  public void restoreExpansionState() {
    WiredTreeModel model = getModel();
    if (model == null) return;
    WiredTreeNode root = model.getRoot();
    if (root == null) return;
    Enumeration<WiredTreeNode> ee = root.postorderEnumeration();
    while (ee.hasMoreElements()) {
      WiredTreeNode node = ee.nextElement();
      TreePath path = new TreePath(node.getPath());
      if (!node.isLeaf()) {
        if (node.isExpanded()) {
          expandPath(path);
        } else {
          collapsePath(path);
        }
      }
    }
  }

  public void setCellRenderer(TreeCellRenderer renderer) {
    super.setCellRenderer(new WiredTreeCellRenderer(this, renderer));
  }

// -- Getters : ----------------------------------------------------------------

  public WiredTreeModel getModel() {
    return (WiredTreeModel) treeModel;
  }

}
