package hep.wired.util.tree;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Font;
import java.awt.Insets;
import javax.swing.Box;
import javax.swing.Icon;
import javax.swing.JCheckBox;
import javax.swing.JPanel;
import javax.swing.JTree;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.TreeCellRenderer;

/**
 * 
 * @author Dmitry Onoprienko
 * @version $Id: $
 */
public class WiredTreeCellRenderer extends JPanel implements TreeCellRenderer {

// -- Private parts : ----------------------------------------------------------

  protected WiredTreeNode _node;
  protected TreeCellRenderer _renderer;
  protected Component _component;
  protected BranchCheckBox _branchBox;
  protected Component _strut;
  protected JCheckBox _nodeBox;

// -- Construction : -----------------------------------------------------------

  public WiredTreeCellRenderer(WiredTree tree) {
    this(tree, new NoIconTreeCellRenderer());
  }

  public WiredTreeCellRenderer(WiredTree tree, TreeCellRenderer renderer) {

    setOpaque(false);
    setLayout(new BorderLayout());

    this._renderer = renderer;
    this._component = renderer.getTreeCellRendererComponent(tree, "", true, true, true, 0, true);

    add(BorderLayout.CENTER, _component);

    _branchBox = new BranchCheckBox();
    _branchBox.setMargin(new Insets(0, 0, 0, 0));
    _branchBox.setBorderPaintedFlat(true);
    _branchBox.setOpaque(false);
    _branchBox.setToolTipText("Branch visibility.");

    _nodeBox = new JCheckBox();
    _nodeBox.setMargin(new Insets(0, 0, 0, 0));
    _nodeBox.setBorderPaintedFlat(true);
    _nodeBox.setOpaque(false);
    _nodeBox.setToolTipText("Node visibility.");

    Box box = Box.createHorizontalBox();
    box.add(_branchBox);
    _strut = Box.createHorizontalStrut(3);
    box.add(_strut);
    box.add(_nodeBox);

    add(BorderLayout.WEST, box);
  }

// -- Implementing TreeCellRenderer : ------------------------------------------

  public Component getTreeCellRendererComponent(JTree tree, Object value,
          boolean selected, boolean expanded, boolean leaf, int row, boolean hasFocus) {

    _node = (WiredTreeNode) value;

    _branchBox.setState(_node.getBranchState());
    _nodeBox.setSelected(_node.isMarkedNode());

    _renderer.getTreeCellRendererComponent(tree, _node.getPayload(), _node.isSelected(), expanded, leaf, row, hasFocus);

    _branchBox.setVisible(!leaf);
    _strut.setVisible(!leaf);

    int style = (_node.hasSelectedDecendents() && ! expanded) ? Font.BOLD : Font.PLAIN;
    _component.setFont(_component.getFont().deriveFont(style));

//    setEnabled(_component.isEnabled());
    boolean show = ! (_node.isHidden() || _node.isInUnmarkedBranch());
    _nodeBox.setEnabled(show);
    _component.setEnabled(show);
    invalidate();
    return this;
  }

//  public void setEnabled(boolean state) {
//    super.setEnabled(state);
//    _branchBox.setEnabled(state);
//    _strut.setEnabled(state);
//    _nodeBox.setEnabled(state);
//    _component.setEnabled(state);
//  }

// -- Do I really need this class ? --------------------------------------------

  public static class NoIconTreeCellRenderer extends DefaultTreeCellRenderer {
    public Icon getOpenIcon() {
      return null;
    }
    public Icon getClosedIcon() {
      return null;
    }
    public Icon getLeafIcon() {
      return null;
    }
  }

// -----------------------------------------------------------------------------

}
