package hep.wired.util.tree.test;

import hep.wired.util.tree.WiredTreeNode;
import hep.wired.util.tree.WiredTree;
import hep.wired.util.tree.WiredTreeModel;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.UIManager;
import javax.swing.tree.TreePath;

/**
 * 
 * @author Mark Donszelmann
 * @version $Id: WiredTreeTest.java 8598 2006-08-14 20:38:02Z duns $
 */
public class WiredTreeTest extends JPanel {

    public WiredTreeTest() {

        setLayout(new BorderLayout());

        WiredTreeNode root = new WiredTreeNode("Root");

        WiredTreeNode geometry = new WiredTreeNode("Geometry");
        root.add(geometry);
        WiredTreeNode box1 = new WiredTreeNode("Box1");
        geometry.add(box1);
        box1.add(new WiredTreeNode("Panel1", WiredTreeNode.Flag.MARKED_BRANCH));
        box1.add(new WiredTreeNode("Panel2"));
        box1.add(new WiredTreeNode("Panel3", WiredTreeNode.Flag.MARKED_BRANCH));

        WiredTreeNode box2 = new WiredTreeNode("Box2");
        geometry.add(box2);
        box2.add(new WiredTreeNode("PanelA"));
        box2.add(new WiredTreeNode("PanelB"));

        WiredTreeNode event = new WiredTreeNode("Event");
        root.add(event);

        final WiredTree tree = new WiredTree(root);

        tree.setRootVisible(true);
        tree.setShowsRootHandles(true);
        add(new JScrollPane(tree), BorderLayout.CENTER);

        JButton button = new JButton("Print List");
        add(button, BorderLayout.SOUTH);

        button.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent event) {
                System.out.println("Tree Expansion state: ");
                WiredTreeModel.TreeVisitor v  = new WiredTreeModel.TreeVisitor() {
                  public boolean visit(TreePath path) {
                    System.out.println(path +" "+ tree.hasBeenExpanded(path));
                    return true;
                  }
                };
                v.visitPaths(tree);
                System.out.println("-------------");
            }
        });
    }

    public static void main(String[] args) {

        try {
            String plaf = UIManager.getSystemLookAndFeelClassName();
//            System.out.println("LF: "+ plaf);
            UIManager.setLookAndFeel(plaf);
//            UIManager.setLookAndFeel(plaf);
        } catch (Exception e) {
            System.out.println("Error loading Look and Feel");
        }

        JFrame frame = new JFrame("Wired Tree Test");

        frame.getContentPane().setLayout(new GridLayout(1, 1));
        frame.getContentPane().add(new WiredTreeTest());
        frame.setSize(200, 400);
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        frame.setVisible(true);
    }
}
